// This is a test program for swap_vector extension to std::vector .
// Due to the current implementation it assumes swap_vector is also in
// the std namespace.
// Change the value of USE_STD_VECTOR to 1 for std::vector in the test;
// a value of 0 uses swap_vector instead of std::vector .
// There are 2 test programs controlled by the "#if 0 {or 1}" at the
// start of main(). The first one plays around with a vector of strings.
// The second one exercises a vector of vectors of strings and expects
// a text file as stdin (source code is fine). All output goes to stdout
// apart form stats which go to stderr.
//
// Douglas Gilbert, 2002/03/14


#include <iostream>
#include <string>
#include <ctype.h>
#include <stdio.h>
#include <swap_vector>
#include <vector>

// Next two defines control 4 possible tests:
//   - simple, using swap_vector [ 0 0 ]
//   - simple, using std::vector [ 0 1 ]
//   - longer, reading text from stdin, using swap_vector [ 1 0 ]
//   - longer, reading text from stdin, using std::vector [ 1 1 ]
// The longer test may well take text redirected from a file.
#define READ_STDIN_TEST 0
#define USE_STD_VECTOR 0

#define SIMPLE_TEST (! READ_STDIN_TEST)

#if USE_STD_VECTOR
#define VECTOR vector
#define PUSH_BACK push_back
#define INSERT insert
#define ASSIGN assign
#else
#define VECTOR swap_vector
#define PUSH_BACK swap_push_back
#define INSERT swap_insert
#define ASSIGN swap_assign
#endif
#define STD_VECTOR vector


using std::swap_vector;
using std::vector;
using std::string;
using std::cin;
using std::cout;
using std::endl;


// Yes, one should not derive from std::string. This class has static members
// that count invocations and sum string lengths seen by copy constructors
// and destructors.
class MyString : public std::string {
public:
    MyString() { ++def_ctors; }
    MyString(const char * cp) : std::string(cp) { ++cs_ctors; }
    MyString(const MyString & ms) : std::string(ms) { 
    	++copy_ctors; copy_ctors_sz += ms.size(); }
    MyString(const std::string & s) : std::string(s) { ++std_s_ctors; }
    ~MyString() { ++dtors; dtors_sz += size(); }
    MyString & operator=(const  MyString & ms) { ++assigns; 
    	return (MyString &) std::string::operator=(ms); }
    void swap(MyString & o) { ++swaps; std::string::swap((std::string &)o); }

    static void printStats(std::ostream & o) {
    	o << "ctors: def=" << def_ctors
    	  << " std_s=" << std_s_ctors
    	  << " cs=" << cs_ctors
    	  << " copy=" << copy_ctors
    	  << "(" << copy_ctors_sz
    	  << ");  dtors=" << dtors
    	  << "(" << dtors_sz
    	  << ")  assigns=" << assigns
    	  << " swaps=" << swaps << std::endl;
    }

private:
    static int def_ctors;
    static int std_s_ctors;
    static int cs_ctors;
    static int copy_ctors;
    static int copy_ctors_sz;
    static int dtors;
    static int dtors_sz;
    static int assigns;
    static int swaps;
};

int MyString::def_ctors = 0;
int MyString::std_s_ctors = 0;
int MyString::cs_ctors = 0;
int MyString::copy_ctors = 0;
int MyString::copy_ctors_sz = 0;
int MyString::dtors = 0;
int MyString::dtors_sz = 0;
int MyString::assigns = 0;
int MyString::swaps = 0;


// Following function borrowed from Koenig+Moo's book

STD_VECTOR <MyString> split(const string& s)
{
    STD_VECTOR <MyString> ret;
    typedef string::size_type string_size;
    string_size i = 0;

    // invariant: we have processed characters `['original value of `i', `i)'
    while (i != s.size()) {
        // ignore leading blanks
        while (i != s.size() && isspace(s[i]))
            ++i;

        // find end of next word
        string_size j = i;
        // invariant: none of the characters in range `['original `j', current `j)' is a space
        while (j != s.size() && !isspace(s[j]))
            ++j;

        // if we found some nonwhitespace characters
        if (i != j) {
            // copy from `s' starting at `i' and taking `j' `\-' `i' chars
            ret.push_back(s.substr(i, j - i));
            i = j;
        }

    }
    return ret;
}

int main()
{
#if SIMPLE_TEST
    char buff[256];

    for (int k = 0; k < 1; ++k) {
	VECTOR <MyString> vi, vi2;
	MyString s("hello this is a test, "), t;
	s += s;
	s += s;

	for (int j = 4; j >= 0; --j) {
	    sprintf(buff, "%d", j);
	    // std::cout << buff << std::endl;
	    t = s + buff;
	    vi. PUSH_BACK (t);
	}
    #if 1
	std::cout << "insert" << std::endl;
	t = "before begin";
	vi. INSERT (vi.begin(), t);
	vi2. insert (vi2.end(), vi.begin(), vi.end());
	std::cout << "I: vector2= ";
	for (size_t k = 0; k < vi2.size(); ++k)
		std::cout << vi2[k] << " ";
	std::cout << std::endl;
	vi.erase(vi.begin(), vi.end() - 2);
	std::cout << "II: vector= ";
	for (size_t k = 0; k < vi.size(); ++k)
		std::cout << vi[k] << " ";
	std::cout << std::endl;
	vi2. ASSIGN (vi.begin(), vi.end());
	std::cout << "III: vector2= ";
	for (size_t k = 0; k < vi2.size(); ++k)
		std::cout << vi2[k] << " ";
	std::cout << std::endl;
    #endif
    }
#else
    {
	string s;
	VECTOR < VECTOR <MyString> > acc_v;
	VECTOR <MyString> vv;

	while (std::getline(cin, s)) {
	    STD_VECTOR <MyString> v = split(s);

	    // Note that the following ASSIGN is swapping/assigning the 
	    // contents of std::vector<MyString> with a swap_vector<MyString>
	    // (if VECTOR is thus defined). Pretty powerful.
#if 1	    
	    vv. ASSIGN (v.begin(), v.end()); 
#else
	    vv.erase(vv.begin(), vv.end());
	    vv. INSERT (vv.begin(), v.begin(), v.end());
#endif
	    acc_v. INSERT (acc_v.begin(), vv);
	}
	for (size_t k = 0; k < acc_v.size(); ++k) {
	    const VECTOR <MyString> & crv(acc_v.at(k));

	    for (size_t j = 0; j < crv.size(); ++j) {
		cout << crv[j];
		if (7 == (j % 8))
		    cout << endl;
		else
		    cout << " ";
	    }
	    cout << endl;
	}
    }
#endif
    MyString::printStats(std::cerr);
    return 0;
}
